/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package ChatProgramm.model;

import ChatProgramm.controller.Nachricht;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.Flow;
import java.util.concurrent.Flow.Subscriber;
import java.util.concurrent.SubmissionPublisher;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author ahren
 */
public abstract class Transmitter implements Runnable, Subscriber<Nachricht> {

    static final int timeout = 10000;
    protected static final int PORT = 35000;
    protected static final String IP = "127.0.0.1";

    private static Logger lg = Logger.getLogger("netz");

    protected Socket socket;
    protected BufferedReader reader;
    protected PrintWriter writer;

    private SubmissionPublisher<Nachricht> textPublisher;
    private Thread thd;
    private boolean laufend;

    public Transmitter() {
        socket = new Socket();
    }

    public abstract void connect() throws IOException;

    public void addWertSubscription(Subscriber<Nachricht> subscriber) {
        textPublisher.subscribe(subscriber);
    }

    public void initIO() {
        try {
            lg.info("Initialisiere reader und writer");
            InputStream is = socket.getInputStream();
            OutputStream os = socket.getOutputStream();

            InputStreamReader isr = new InputStreamReader(is, "UTF-8");
            OutputStreamWriter osr = new OutputStreamWriter(os, "UTF-8");

            reader = new BufferedReader(isr);
            writer = new PrintWriter(osr);
            lg.info("Initialisierung abgeschlossen");

            lg.info("Warte auf Nachricht");

        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(Transmitter.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Transmitter.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void send(Nachricht nachricht) {
        lg.info("Nachricht wird gesendet");
        writer.println(nachricht.getNachricht());
        writer.flush();

        lg.info("Nachricht wird angezeigt");
        textPublisher.submit(nachricht);
    }

//    public void startServer() {
//        laufend = true;
//        if (thd == null) {
//            thd = new Thread(this);
//            thd.start();
//        }
//
//        synchronized (thd) {
//            thd.notify();
//        }
//
//    }

    @Override
    public void run() {
        try {
            Nachricht nachricht = new Nachricht(reader.readLine());
            textPublisher.submit(nachricht);
        } catch (IOException ex) {
            Logger.getLogger(Transmitter.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public void onSubscribe(Flow.Subscription subscription) {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void onError(Throwable throwable) {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void onComplete() {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void onNext(Nachricht item) {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

}
